package fr.username404.snowygui.utils

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.BufferBuilder
import com.mojang.blaze3d.vertex.DefaultVertexFormat
import com.mojang.blaze3d.vertex.Tesselator
import com.mojang.blaze3d.vertex.VertexConsumer
import fr.username404.snowygui.gui.feature.Colors
import fr.username404.snowygui.gui.hextoRGB
import org.lwjgl.opengl.GL20

object RenderingUtil {
    @JvmField val tessellator: Tesselator = Tesselator.getInstance()
    @JvmField val buffer: BufferBuilder = tessellator.builder
    @JvmStatic
    fun VertexConsumer.colorIt(color: Int, opacity: Float = 1F): VertexConsumer = hextoRGB(color).run {
        color(get(0), get(1), get(2), opacity)
    }
    fun VertexConsumer.colorEnd(color: Int, opacity: Float = 1F) = colorIt(color, opacity).endVertex()
    fun prepareDraw() {
        RenderSystem.disableTexture()
        RenderSystem.enableBlend()
        RenderSystem.defaultBlendFunc()
    }
    fun endDraw() {
        RenderSystem.enableTexture()
        RenderSystem.disableBlend()
    }
    fun drawRectangle(
        x: Double, y: Double, height: Int, width: Int,
        color: Int = Colors.TRANSPARENT(), opacity: Float = 1F
    ): Unit = buffer.run {
        fun VertexConsumer.colorEnd() = colorEnd(color, opacity)
        begin(GL20.GL_QUADS, DefaultVertexFormat.POSITION_COLOR)
        vertex(x, y + height, 0.0).colorEnd()
        vertex(x + width, y + height, 0.0).colorEnd()
        vertex(x + width, y, 0.0).colorEnd()
        vertex(x, y, 0.0).colorEnd()
        tessellator.end()
    }
}
